using System.ComponentModel.DataAnnotations;
using CodeLiturgy.Data.Application.Users;

namespace CodeLiturgy.Data.Auth.Context.Users;

/// <summary>
/// 
/// </summary>
public class RegisterRequest
{
    /// <summary>
    /// Email
    /// </summary>
    [Required]
    [EmailAddress]
    [Display(Name = "Email")]
    public string Email { get; set; }

    /// <summary>
    /// Password
    /// </summary>
    [Required]
    [StringLength(100, ErrorMessage = "The {0} must be at least {2} characters long.", MinimumLength = 6)]
    [DataType(DataType.Password)]
    [Display(Name = "Password")]
    public string Password { get; set; }
    /// <summary>
    /// Username 
    /// </summary>
    public string Username { get; set; }

    /// <summary>
    /// ConfirmPassword
    /// </summary>
    [DataType(DataType.Password)]
    [Display(Name = "Confirm password")]
    [Compare("Password", ErrorMessage = "The password and confirmation password do not match.")]
    public string ConfirmPassword { get; set; }
    
    
    /// <summary>
    /// ConfirmPassword
    /// </summary>
    [DataType(DataType.PhoneNumber)]
    [Display(Name = "Phone Number")]
    public string PhoneNumber { get; set; }
    
    

    /// <summary>
    /// Convert RegisterViewModel to ApplicationUser
    /// </summary>
    /// <returns></returns>
    public ApplicationUser ToUser()
    {
        var newUser = new ApplicationUser();
        newUser.Email = Email;
        newUser.PasswordHash = Password;
        newUser.UserName = Username;
        newUser.PhoneNumber = PhoneNumber;
        return newUser;
    }
}